package POE::Component::Bundle::WebDevelopment;

use strict;
use warnings;

our $VERSION = '1.001002'; # VERSION

1;

__END__

=for stopwords DOCTYPEs Ipsum Lorem bots html validator

=encoding utf8

=head1 NAME

POE::Component::Bundle::WebDevelopment - a collection of PoCos useful for Web Development

=head1 CONTENTS

This distribution provides the following
L<POE::Component> modules:

=over 4

=item * EDIT AFTER THE FIRST UPLOAD, AS IT WILL BE EASIER TO JUST COPY PASTE THE WHOLE LIST!

=back

=head1 EXAMPLES

This distribution contains C<examples/> directory, with usage examples
of each component

=head1 REPOSITORY

Fork this module on GitHub:
L<https://github.com/zoffixznet/POE-Component-Bundle-WebDevelopment>

=head1 BUGS

To report bugs or request features, please use
L<https://github.com/zoffixznet/POE-Component-Bundle-WebDevelopment/issues>

If you can't access GitHub, you can email your request
to C<bug-POE-Component-Bundle-WebDevelopment at rt.cpan.org>

=head1 AUTHOR

Zoffix Znet <zoffix at cpan.org>
(L<http://zoffix.com/>, L<http://haslayout.net/>)

=head1 LICENSE

You can use and distribute this module under the same terms as Perl itself.
See the C<LICENSE> file included in this distribution for complete
details.

=cut